(use cairo)

(include "date.scm")
(include "paper.scm")
(include "anniversaries.scm")
(import date paper)

(define heading-size 16)
(define day-size 6)
(define holiday-size 4)

(define font "EBGaramond")

;; These are in millimeters
;; margins on the page
(define left-margin 10)
(define top-margin 30)

;; space between headings and days
(define header-spacing 10)

;; size of one column
(define col-size 85)
;; space between month columns
(define col-spacing 10)

;; The box sizes for a day (without date)
(define box-width 65)
(define box-height 10)

;; space between two days in one column
(define line-spacing 2)

(define year 2017)

(define (holidays year)
  (append (state-holidays (list year))
          (roman-catholic-holidays (list year))))

(define (text c x y txt #!key (font-size day-size) (color '(0 0 0 1)) emphasis?)
  (cairo-save c)
  (when emphasis?
        (cairo-select-font-face c font CAIRO_FONT_SLANT_ITALIC CAIRO_FONT_WEIGHT_NORMAL))
  (apply cairo-set-source-rgba (cons c color))
  (cairo-set-font-size c font-size)
  (cairo-move-to c x y)
  (cairo-show-text c txt)
  (cairo-stroke c)
  (cairo-restore c))

(define (text-width text #!key (font-size heading-size))
  (define ex (make-cairo-text-extents-type))
  (cairo-set-font-size c font-size)
  (cairo-text-extents c text ex)
  (values (cairo-text-extents-width ex)
          (cairo-text-extents-height ex)))

(define (random-color c)
  (define (rc)
    (/ (random 10) 10))
  (let ((r (rc)) (g (rc)) (b (rc)))
    (if (or (< (+ r g b) 1)
            (> (+ r g b) 2))
        (random-color c)
        (list r g b))))

(define (print-month c x y month)
  (let* ((month (list year month))
         (days (days-in-month month))
         (padding (if (< days 31)
                      (* (+ box-height line-spacing) (- 31 days))
                      0))
         (h (holidays year)))
    (text c x y (month->string month) font-size: heading-size color: `(,@(random-color c) 1))
    (let loop ((y (+ y header-spacing padding))
               (d 1))
      (let ((full-date `(,@month ,d))
            (color (random-color c)))
        (unless (> d days)
                (cairo-save c)
                (when (weekend? full-date)
                      (cairo-set-source-rgba c 1 0 0 1))
                (text c x (+ y (* box-height (/ 2 3))) (sprintf "~a ~a" (day->short-string full-date) d) color: (if (weekend? full-date)
                                                                                                                   '(1 0 0 1)
                                                                                                                   '(0 0 0 1)))
                (cairo-set-line-width c 0.2)
                (cairo-rotate c (* (/ cairo-pi 180) (/ (- 2 (random 4)) 4)))
                (cairo-rectangle c (+ x (- col-size box-width)) y box-width box-height)
                (cairo-set-source-rgba c
                                       (first color)
                                       (second color)
                                       (third color)
                 (if (weekend? full-date) 0.8 0.2))
                (cairo-fill c)
                (cond ((filter-map (lambda (e) (if (equal? (car e) full-date)
                                                   (cdr e)
                                                   #f)) h)
                       => (lambda (d)
                            (let ((t (string-intersperse d " / ") ))
                              (text c
                                    (+ x (- col-size (text-width t font-size: holiday-size) 1))
                                    (+ y (* box-height (/ 5 6)))
                                    t
                                    font-size: holiday-size)))))
                (cond ((filter-map (lambda (e) (if (equal? (cdar e) (cdr full-date))
                                                   (render-anniversary full-date e)
                                                   #f))
                                   (anniversaries year))
                       =>
                       (lambda (d)
                            (let ((t (string-intersperse d " / ") ))
                              (text c
                                    (+ x (- col-size (text-width t font-size: holiday-size) 1))
                                    (+ y (* box-height (/ 1 3)))
                                    t
                                    font-size: holiday-size
                                    emphasis?: #t)))))
#;
                (text c
                (+ x (- col-size box-width))
                (+ y (* box-height (/ 1 3)))
                (->string (moon-phase full-date))
                font-size: holiday-size)
                (cairo-restore c)
                (loop (+ y box-height line-spacing) (add1 d)))))))

;; -------------------------- main stuff --------------------------

(define s (apply cairo-pdf-surface-create (cons (sprintf "calendar-~a.pdf" year) (paper-dimensions 'a3))))
(define c (cairo-create s))

(cairo-select-font-face c font CAIRO_FONT_SLANT_NORMAL CAIRO_FONT_WEIGHT_NORMAL)
(cairo-set-font-size c heading-size)

;; use millimeters as units from here on
(cairo-scale c (/ 72 25.4) (/ 72 25.4))

(text c (- 297 (/ col-size 3) col-spacing) (* top-margin (/ 2 3)) (->string year) font-size: heading-size color: `(,@(random-color c) 1))
(let col-loop ((x left-margin)
               (y top-margin)
               (m 1))
  (unless (> m 12)
          (print-month c x y m)
          (cond ((and (< m 12) (zero? (modulo m 3)))
;                 (cairo-surface-write-to-png s (sprintf "cal-~a.png" (/ m 3)))
                 (cairo-surface-show-page s)
                 (text c (- 297 (/ col-size 3) col-spacing) (* top-margin (/ 2 3)) (->string year) font-size: heading-size color: `(,@(random-color c) 1))
                 (col-loop left-margin top-margin (add1 m)))
                (else
                 (col-loop (+ x col-size col-spacing) y (add1 m))))))

(cairo-surface-show-page s)
(cairo-destroy c)
(cairo-surface-finish s)
(cairo-surface-destroy s)
